/*
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License. 
 * 
 * The Original Code is pilot-mailsync.
 * 
 * The Initial Developer of the Original Code is Hollis Blanchard
 * <hollis@austin.rr.com>. Portions created by Hollis Blanchard are Copyright
 * (C) 2001 Hollis Blanchard. All Rights Reserved.
 * 
 * Contributor(s):
 * Edgardo Garcia Hoeffler <Edgardo.Garcia-Hoeffler@itau.com.ar>
 */

#include <glib.h>
#include <gnome.h>

#include <pi-source.h>
#include <pi-socket.h>
#include <pi-mail.h>
#include <pi-dlp.h>
#include <pi-version.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <utime.h>
#include <unistd.h>
#include <pwd.h>
#include <signal.h>

#include <gpilotd/gnome-pilot-conduit.h>
#include <gpilotd/gnome-pilot-conduit-standard.h>
#include "gnome_mailsync_conduit.h"

#include "pilot-mailsync.h"

#define CONDUIT_VERSION "0.10"

/*#define EC_DEBUG */
#ifdef EC_DEBUG
#define LOG(format,args...) g_log (G_LOG_DOMAIN, \
                                   G_LOG_LEVEL_MESSAGE, \
                                   "email: "##format, ##args)
#else
#define LOG(format,args...)
#endif

GnomePilotConduit *conduit_get_gpilot_conduit( guint32 pilotId ) ;
void conduit_destroy_gpilot_conduit( GnomePilotConduit *c );

static void 
load_configuration(ConduitCfg **c,guint32 pilotId) 
{
/* Mas adelante lo sacamos de aca y lo pasamos
	gchar *prefix;

	g_assert(c!=NULL);
	*c = g_new0(ConduitCfg,1);
	(*c)->child = -1;

	prefix = g_strdup_printf("/gnome-pilot.d/email-conduit/Pilot_%u/",pilotId);
  
	gnome_config_push_prefix(prefix);
	(*c)->sendmail = gnome_config_get_string( "sendmail=/usr/lib/sendmail -t -i");
	(*c)->fromAddr = gnome_config_get_string( "from_address" );
	(*c)->sendAction = gnome_config_get_string( "send_action=file");
	(*c)->mhDirectory = gnome_config_get_string( "mh_directory" );
	(*c)->mboxFile = gnome_config_get_string ( "mbox_file" );
	(*c)->receiveAction = gnome_config_get_string( "receive_action=file" );
	gnome_config_pop_prefix();

	(*c)->pilotId = pilotId;
	g_free(prefix);
*/
return;
}

/** this method frees all data from the conduit config */
static void 
destroy_configuration(ConduitCfg **c) 
{
	g_assert(c!=NULL);
	g_assert(*c!=NULL);
	g_free( (*c)->sendmail );
	g_free( (*c)->fromAddr );
	g_free( (*c)->sendAction );
	g_free( (*c)->mhDirectory );
	g_free( (*c)->mboxFile );
	g_free( (*c)->receiveAction );
	g_free(*c);
	*c = NULL;
}



static gint synchronize( GnomePilotConduit *c, GnomePilotDBInfo *dbi ) 
{
    return gnomeConduitsMailSync(dbi->pilot_socket);
}

gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi ) {
    return( synchronize( c, dbi ) );
}

GnomePilotConduit *conduit_get_gpilot_conduit( guint32 pilotId ) 
{
  GtkObject *retval;
  ConduitCfg *cc;

  retval = gnome_pilot_conduit_standard_new("MailDB",0x6d61696c, NULL);

  g_assert(retval != NULL);
  /*
  gtk_signal_connect(retval, "copy_from_pilot", (GtkSignalFunc)copy_from_pilot ,NULL);
    gtk_signal_connect(retval, "copy_to_pilot", (GtkSignalFunc) ,NULL);
    gtk_signal_connect(retval, "merge_to_pilot", (GtkSignalFunc) ,NULL);
  gtk_signal_connect(retval, "merge_from_pilot", (GtkSignalFunc)synchronize ,NULL);
  */
  gtk_signal_connect(retval, "synchronize", (GtkSignalFunc)synchronize ,NULL);

  load_configuration(&cc, pilotId );
  gtk_object_set_data(retval,"conduit_config",(gpointer)cc);
  return GNOME_PILOT_CONDUIT(retval); 
}


void conduit_destroy_gpilot_conduit( GnomePilotConduit *c ) 
{
/*  ConduitCfg *cc;
  
  cc = GET_CONFIG(c);
  destroy_configuration( &cc );*/
  gtk_object_destroy(GTK_OBJECT(c));
}
