/*
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/ 
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License. 
 * 
 * The Original Code is pilot-mailsync.
 * 
 * The Initial Developer of the Original Code is Hollis Blanchard
 * <hollis@austin.rr.com>. Portions created by Hollis Blanchard are Copyright
 * (C) 2001 Hollis Blanchard. All Rights Reserved.
 * 
 * Contributor(s):
 * 
 */


/* doubling array to store a copy of Palm mail */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mailstore.h"

static struct Mail **store;
static int storesize;
static int used; /* indexes to the first open slot */


/* initStore
 *
 * start us off with our doubling array.
 */
void initStore(void)
{
	storesize = 10;
	store = (struct Mail **)malloc(sizeof(struct Mail) * storesize);
}

/* emptyStore
 *
 * release the structs.
 */
void emptyStore(void)
{
	while (used--) {
		free(store[used]);
	}
	storesize=0;
}

/* storeMsg
 *
 * add Palm Mail struct ptr to our array
 */
int storeMsg(struct Mail *msg)
{
	if (!msg)
		return 0;

	if (used >= storesize) {
		storesize *= 2;
#ifdef DEBUG
		printf("growing mail store to %i\n", storesize);
#endif
		store = (struct Mail **)realloc(store, sizeof(struct Mail) * storesize);
	}

	store[used++] = msg;
	return 1;
}

/* cmpstr
 * 
 * NULL-safe strcmp with a twist:
 *   returns True (same) or False (different)
 */
static int cmpstr(char *a, char *b)
{
	if (!a && !b)
		/* both NULL */
		return 1;

	if ((a && !b) || (!a && b))
		/* one was NULL, the other wasn't */
		return 0;

	/* both non-NULL, safe for strcmp */
	return !strcmp(a, b);
}

/* search for this Mail in our storage */
struct Mail *findMsg(struct Mail *msg)
{
	int i;

	if (!msg)
		return NULL;

	for (i=0; i<used; i++) {
		struct Mail *cur = store[i];

		/* check date first because it's probably the most unique. Also, it's
		 * undoubtably a faster compare.
		 */

		/* apparently tm_sec is not stored Palm-side (it's always 0 when read
		 * back)
		 */
		if (
				(cur->date.tm_min != msg->date.tm_min) ||
				(cur->date.tm_hour != msg->date.tm_hour) ||
				(cur->date.tm_mday != msg->date.tm_mday) ||
				(cur->date.tm_mon != msg->date.tm_mon) ||
				(cur->date.tm_year != msg->date.tm_year)
			)
			continue;

		if (! cmpstr(cur->subject, msg->subject))
			continue;
		if (! cmpstr(cur->from, msg->from))
			continue;
		if (! cmpstr(cur->cc, msg->cc))
			continue;
		if (! cmpstr(cur->bcc, msg->bcc))
			continue;
		if (! cmpstr(cur->replyTo, msg->replyTo))
			continue;
		if (! cmpstr(cur->sentTo, msg->sentTo))
			continue;
		/* almost always the same, I would guess: */
		if (! cmpstr(cur->to, msg->to))
			continue;

		/* if we made it here all the headers match */
		return cur;
	}

	/* yes, we searched all n emails and didn't find it */
	return NULL;
}
